/**********************************************************************//**
* @file		AC33Mx064_hal_scu.c
* @brief	Contains all functions support for SCU firmware library on AC33Mx064
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "AC33Mx064_hal_scu.h"


/**********************************************************************//**
 * @brief 		Configure external clock  
 *             Before exteranl crystal enable setting, you have to set PC12,PC13 as XIN,XOUT in PCMR.
 * @param[in]	eosccon 	Function mode, should be:
 * 					- EXOSC_STOP	   : 0
 * 					- EXOSC_EN    	   : 2
 * 					- EXOSC_EN_DIV2 : 3
 * @return		None
 **********************************************************************/
void HAL_SCU_HSE_ClockConfig(uint8_t eosccon)
{
	uint8_t reg_val8;
	
// enable external clock  
// Before exteranl crystal enable setting, you have to set PC12,PC13 as XIN,XOUT in PCMR.
	reg_val8=SCU->CSCR;
	reg_val8&=~(3<<0); // 
	reg_val8|=(eosccon<<0); // 0,1:Stop External Ctystal oscillator, 2:Enable External Ctystal oscillator, 3:Enable External Ctystal divide by 2
	SCU->CSCR=reg_val8;	
}

/**********************************************************************//**
 * @brief 	  monitoring External oscillator  
 * @param  none
 * @return	status enumeration value:
 *              - ERROR
 *              - SUCCESS
 **********************************************************************/
Status HAL_SCU_HSE_ClockMonitoring(void)
{
	__IO uint32_t StartUpCounter = 0;
	__IO uint16_t status;
	uint32_t i;
		
	// monitoring External oscillator 
	SCU->CMR|=(1<<3); // 0:External oscillator monitoring disabled, 1:External oscillator monitoring enable
	
	/* Wait till MOSC is ready and if timeout is reached exit */
	do 
	{
		status = (SCU->CMR & SCU_CMR_EOSCSTS_Msk);
		StartUpCounter++;  
	} while((StartUpCounter != MOSC_STARTUP_TIMEOUT) && (status != SCU_CMR_EOSCSTS_Msk));	
	
	/* need more Wait for MOSC stable, check XOUT pin oscillation  */	
	for (i=0;i<0x8000;i++)
	{
#ifdef __CC_ARM
		__nop();
#else	  
		asm("NOP");
#endif
	}
	
	status = (SCU->CMR & SCU_CMR_EOSCSTS_Msk);
	if (status != SCU_CMR_EOSCSTS_Msk){
		return ERROR;
	}
	else {
		return SUCCESS;
	}
}

/**********************************************************************//**
 * @brief 		Configure internal ring osc clock  
 * @param[in]	ringosc Function mode, should be:
 * 					- INRINGOSC_STOP		   : 0
 * 					- INRINGOSC_EN    	     : 2
 * 					- INRINGOSC_EN_DIV2  : 3
 * @return		None
 **********************************************************************/
void HAL_SCU_LSI_ClockConfig(uint8_t ringosc)
{
	uint8_t reg_val8;
	
// enable internal clock  
	reg_val8=SCU->CSCR;
	reg_val8&=~(3<<4); // 
	reg_val8|=(ringosc<<4); // 0,1:Stop Internal ring oscillator, 2:Enable Internal ring oscillator, 3:Enable Internal ring divide by 2
	SCU->CSCR=reg_val8;	
}


/**********************************************************************//**
 * @brief 		change system clock control   
 * @param[in]	sysclkcon mode, should be:
 * 					- SC_INRINGOSC		         : 0
 * 					- SC_EXOSC  	               : 6
 * 					- SC_EXOSCPLL              : 7
 * @return		None
 **********************************************************************/
void HAL_SCU_SystemClockChange(uint8_t sysclkcon)
{
	uint8_t reg_val8;
	
	reg_val8=SCU->SCCR;
	reg_val8&=~(7<<0); //
	reg_val8|=sysclkcon;  //FINSEL[2] - 0:IOSC, 1:MOSC(external crystal)
	                                    //MCLKSEL[1:0] -  0,1:Internal sub oscillator(ROSC), 2:PLL bypassed clock, 3:PLL output clock
	SCU->SCCR=reg_val8;	
}


/**********************************************************************//**
 * @brief 		Configure clock out  on PC9
 *       Before this function setting, you have to set PC9 as CLKO in PCMR.
 * @param[in]	divval(=CLKODIV) : 0~15  // CLKO=MCLK/(2*(CLKODIV+1))
 *                   ex) MCLK=20Mh, CLKODIV=4,    20Mhz/ (2*(4+1)) = 2Mhz
 * @param[in]	FunctionalState, should be:
 * 					- DISABLE		   : 0
 * 					- ENABLE    	   : 1
 * @return		None
 **********************************************************************/
void HAL_SCU_ClockOutput(uint8_t divval, FunctionalState endis)
{
	SCU->COR=(endis<<4)|(divval&0xf);      
}

/**********************************************************************//**
 * @brief 	  monitoring External oscillator  
 * @param  none
 * @return	status enumeration value:
 *              - ERROR
 *              - SUCCESS
 **********************************************************************/
Status HAL_SCU_PLL_ClockConfig(FunctionalState pllsetstate, uint8_t selbypass, uint8_t selfin, uint8_t MM, uint8_t NN)
{
	__IO uint32_t StartUpCounter = 0;
	__IO uint16_t status;

	SCU->PLLCON=0;
	if (pllsetstate == DISABLE){		
		return SUCCESS;
	}
	else {
// PLLenable    PLL freq = (FIN or FIN/2) * M / N
		SCU->PLLCON=0
		|(1<<15)   // PLL reset is negated
		|(1<<14)  // PLLEn
		|((selbypass&1)<<13)  //BYPASS 0:FOUT is bypassed as FIN, 1:FOUT is PLL output
		|((selfin&1)<<8)   //FIN predivider 0:FIN divided by 1, 1:FIN divided by 2
		|((MM&0xf)<<4)   //FBCTRL  M=12
		|((NN&0xf)<<0)   //FBCTRL  N=1	
			;
		
	  /* Wait till PLL LOCK is ready and if timeout is reached exit */
		do
		{
			status = (SCU->PLLCON & SCU_PLLCON_LOCK_Msk);
			StartUpCounter++;  
		} while((StartUpCounter != PLL_STARTUP_TIMEOUT) && (status != SCU_PLLCON_LOCK_Msk));	
		
		status = (SCU->PLLCON & SCU_PLLCON_LOCK_Msk);
		if (status != SCU_PLLCON_LOCK_Msk){
			return ERROR;
		}
		else {
			return SUCCESS;
		}
	}
}


/**********************************************************************//**
 * @brief 		Configure peri clock setting
 * @param[in]	mccrnum: select REG number
 *                     - MCCR1 : 1, * - MCCR2 : 2
 *                     - MCCR3 : 3, * - MCCR4 : 4
 *                     - MCCR5 : 5, * - MCCR6 : 6
 * @param[in]	type : 
 *	                     SYSTICK_TYPE, TRACE_TYPE, PWM0_TYPE, PWM1_TYPE,
 *						WDT_TYPE, TEXT_TYPE, PAD_TYPE, PBD_TYPE, PCD_TYPE,
 *						PDD_TYPE, ETCD_TYPE, ADCC_TYPE	
 * @param[in]	clksrc : 
 *						- CLKSRC_RING_OSC_1M=0,
 *						- CLKSRC_MCLK=4,
 *						- CLKSRC_EOSC=6,	
 *						- CLKSRC_PLL=7	
 * @param[in]	clkdiv : 
 * @return		None
 **********************************************************************/
void HAL_SCU_MiscClockConfig(uint8_t mccrnum, uint8_t type, int8_t clksrc, int8_t clkdiv)
{
	uint32_t tmp;
	
	switch(mccrnum){
		case 1:
			tmp=SCU->MCCR1;
			if (type==SYSTICK_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			else if (type==TRACE_TYPE){
				tmp&=0xF800FFFF;
				tmp|= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));					
			}
			SCU->MCCR1=tmp;
			break;
			
		case 2:
			tmp=SCU->MCCR2;
			if (type==PWM0_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			SCU->MCCR2=tmp;			
			break;
			
		case 3:
			tmp=SCU->MCCR3;
			if (type==WDT_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			else if (type==TEXT_TYPE){
				tmp&=0xF800FFFF;
				tmp|= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));					
			}
			SCU->MCCR3=tmp;			
			break;
			
		case 4:
			tmp=SCU->MCCR4;
			if (type==PAD_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			else if (type==PBD_TYPE){
				tmp&=0xF800FFFF;
				tmp|= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));					
			}
			SCU->MCCR4=tmp;			
			break;
			
		case 5:
			tmp=SCU->MCCR5;
			if (type==PCD_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			else if (type==PDD_TYPE){
				tmp&=0xF800FFFF;
				tmp|= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));					
			}
			SCU->MCCR5=tmp;			
			break;
			
		case 6:
			tmp=SCU->MCCR7;
			if (type==ADCC_TYPE){
				tmp&=0xF800FFFF;
				tmp|= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));					
			}
			SCU->MCCR7=tmp;			
			break;			
	}
}

/**********************************************************************//**
 * @brief 		set LVD level   
 * @param[in]	bodlevel, should be:
 * 					- LVD_LEVEL_1_8 : 0
 * 					- LVD_LEVEL_2_2 : 1
 * 					- LVD_LEVEL_2_7 : 2
 * 					- LVD_LEVEL_4_3 : 3
 * @return		None
 **********************************************************************/
void HAL_SCU_LVD_SetLevel(uint32_t lvdlevel)
{
	uint32_t tmp;
	
	tmp=SCU->LVDCON;
	tmp&=~(3<<8); //
	tmp|=(1<<15) | (lvdlevel <<8); 
	SCU->LVDCON=tmp;	
}

/**********************************************************************//**
 * @brief	 	enable/disable LVD
 * @param[in]	NewState
 * 					- ENABLE  	:Set LVD enable
 * 					- DISABLE 	:Disable LVD
 * @return 		None
 **********************************************************************/
void HAL_SCU_LVDCmd(FunctionalState NewState)
{
	if (NewState == ENABLE)
	{
		SCU->LVDCON |=  (1<<0);
	}
	else
	{
		SCU->LVDCON &= ~(1<<0);
	}
}

/**********************************************************************//**
 * @brief	 	enable/disable WakeUp Source
 * @param[in]	WakeUpSrc, oring is possible:
 * 					- WAKEUP_GPIOD :    (1UL<<11)
 * 					- WAKEUP_GPIOC :    (1UL<<10)
 * 					- WAKEUP_GPIOB :    (1UL<<9)
 * 					- WAKEUP_GPIOA :   (1UL<<8)
 * 					- WAKEUP_WDT    :    (1UL<<1)
 * 					- WAKEUP_LVD    :    (1UL<<0)
 * @param[in]	NewState
 * 					- ENABLE  	:Set WakeUp Source enable
 * 					- DISABLE 	:Disable WakeUp Source
 * @return 		None
 **********************************************************************/
void HAL_SCU_WakeUpSRCCmd(uint32_t WakeUpSrc, FunctionalState NewState)
{
	if (NewState == ENABLE)
	{
		SCU->WUER |=  (WakeUpSrc);
	}
	else
	{
		SCU->WUER &= ~(WakeUpSrc);
	}
}

/*********************************************************************//**
 * @brief 		Get current value of WakeUpSRC Status.
 * @param[in]	None.
 * @return		Current value of WakeUpSRC Status register.
 *********************************************************************/
uint32_t HAL_SCU_GetWakeUpSRCStatus(void)
{
	return (SCU->WUSR);
}

/**********************************************************************//**
 * @brief	 	enable/disable WakeUp Source
 * @param[in]	WakeUpSrc
 * @return 		None
 **********************************************************************/
void HAL_SCU_ClearWakeUpSRC(uint32_t WakeUpStatus)
{
	SCU->WUSR = WakeUpStatus;
}

/*********************************************************************//**
 * @brief 		Get current value of NMI Status.
 * @param[in]	None.
 * @return		Current value of NMI Status register.
 *********************************************************************/
uint32_t HAL_SCU_GetNMIStatus(void)
{
	return (SCU->NMIR);
}

/**********************************************************************//**
 * @brief	 	enable/disable NMI Source
 * @param[in]	nmisrc
 * @return 		None
 **********************************************************************/
void HAL_SCU_SetNMI(uint32_t nmisrc)
{
	SCU->NMIR = (0xA32CUL<<16) | (nmisrc & 0x1F);
}


/* --------------------------------- End Of File ------------------------------ */
