/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void ADC_Configure(void);
void ADC_InterruptRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);
void ADC0_IRQHandler_IT(void);
/* Private variables ---------------------------------------------------------*/
static uint32_t fflag;

static uint32_t adcirq_single;
static uint32_t adcirq_seq;
static uint32_t adcirq_trigger;

static ADC_CFG_Type AD0_config;

static TIMER_PERIODICCFG_Type T0_Config;
static uint32_t count;
static uint32_t tmp;

static const uint8_t menu[] =
"************************************************\n\r"
" ADC demo \n\r"
"\t - MCU: AC33M4064 \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 115200 bps \n\r"
" Using ADC0 in sequence mode \n\r"
"************************************************\n\r";
/* Public Function ------------------------------------------------------------*/
/**********************************************************************
 * @brief		 ADC0 interrupt handler
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void ADC0_IRQHandler_IT(void)
{
	uint16_t status;
	status = HAL_ADC_GetStatus(AD0);

	if (status & ADC_STAT_SINGLE){
		HAL_ADC_ClearStatus(AD0, ADC_STAT_SINGLE);	
		adcirq_single++;
	}
	if (status & ADC_STAT_SEQ){
		fflag=1;
		HAL_ADC_ClearStatus(AD0, ADC_STAT_SEQ);	
		adcirq_seq++;
	}
	if (status & ADC_STAT_TRIGGER){
		HAL_ADC_ClearStatus(AD0, ADC_STAT_TRIGGER);	
		adcirq_trigger++;
	}	
}


/**********************************************************************
 * @brief		 TIMER0 interrupt handler
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER0_IRQHandler_IT(void)
{
	if ((HAL_TIMER_GetStatus(T0) & TIMER_SR_MFB) == TIMER_SR_MFB){
		HAL_TIMER_ClearStatus(T0, TIMER_SR_MFB);
		count++;
	}	
}


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}

/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		adcaverage
 * @param[in]	*val 	adc value
 * @return 		adc average value
 **********************************************************************/
uint16_t adcaverage(uint16_t *val)
{
	uint32_t i;
	uint32_t adcavg;
	
	adcavg=0;			
	for (i=4;i<8;i++){	
		adcavg+=(val[i]>>4);
	}
	adcavg>>=2;
	
	return 	(adcavg);
}


/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{
	AD0_config.Mode = ADC_SINGLE_MODE;
	AD0_config.SamplingTime=15;
	AD0_config.SeqCnt = 4;
	AD0_config.RestartEn = 0;
	AD0_config.TrgSel = ADC_TRIGGER_TIMER;
	
	AD0_config.UseClk = ADC_INTERNAL_CLK;
	AD0_config.InClkDiv = 4; // 48Mhz / 4 = 12Mhz 
	HAL_ADC_Init(AD0, &AD0_config);

	HAL_ADC_TriggerlSel(AD0, 0x00000000); //trigger timer0
	HAL_ADC_ChannelSel(AD0, 0x76543210);	//ch0, ch1, ch2, ch3
	
	HAL_ADC_ClearStatus(AD0, 0xff); // clear status
	HAL_ADC_ConfigInterrupt(AD0, ( ADC_INTEN_SEQ | ADC_INTEN_TRIGGER), ENABLE);
	
	NVIC_SetPriority(ADC0_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for ADC0 channel */
  NVIC_EnableIRQ(ADC0_IRQn);
	
	adcirq_single = 0;
	adcirq_seq = 0;
	adcirq_trigger =0;
}


/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	T0_Config.CkSel = PCLK_2; // SystemPeriClock=48Mhz=PCLK  CLOCK_IN = PCLK/2=24Mhz 
	T0_Config.Prescaler = 23;    // TCLK = (CLOCK_IN / (PRS+1))  = 24Mhz / 24 = 1Mhz ->1us

	T0_Config.GRA = (500); 	// adc trigger signal 
	T0_Config.GRB = (10000); // 10msec			
	T0_Config.Cnt=0;		
	T0_Config.StartLevel=START_LOW;
	T0_Config.AdcTrgEn=ENABLE;
	
	HAL_TIMER_Init(T0, PERIODIC_MODE, &T0_Config); // timer0 setting 
	HAL_TIMER_ConfigInterrupt(T0, TIMER_INTCFG_MBIE, ENABLE);

  /* preemption = 1, sub-priority = 1 */
  NVIC_SetPriority(TIMER0_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for TIMER0 channel */
  NVIC_EnableIRQ(TIMER0_IRQn);	
	
	count=0;
	HAL_TIMER_Cmd(T0, ENABLE); // timer start
}


/**********************************************************************
 * @brief		ADC_ContinuousRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_ContinuousRun(void)
{			
	if (AD0_config.RestartEn==0) {
			HAL_ADC_Start(AD0); // start 	
		}
		
		fflag=0;
		while(fflag==0){}

		_DBG("AN0 = "); _DBH16(AD0->DR0>>4); _DBG(" ");
		_DBG("AN1 = "); _DBH16(AD0->DR1>>4); _DBG(" ");
		_DBG("AN2 = "); _DBH16(AD0->DR2>>4); _DBG(" ");
		_DBG("AN3 = "); _DBH16(AD0->DR3>>4); _DBG_(" ");			
				
		for(tmp = 0; tmp < 1000000; tmp++);
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
 
	/*Configure adc peripheral*/
	ADC_Configure();

	/*Configure timer peripheral*/
	TIMER_Configure();
	
	/* Enable IRQ Interrupts */
	__enable_irq();
	
	while(1)
	{
		/*ADC Continuous Test start*/
		ADC_ContinuousRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
