/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Tx                               T0
#define TIMERx_IRQn              TIMER0_IRQn

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TIMER_Configure(void);
void TIMER_OneShotRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
static const uint8_t menu[] =
"************************************************\n\r"
"Timer One Shot demo \n\r"
"\t - MCU: AC33M4064 \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 115200 bps \n\r"
" Using Timer 0 in One Shot mode\n\r"
"************************************************\n\r";

static TIMER_ONESHOTCFG_Type  T0_Config;

/* Public Function ------------------------------------------------------------*/


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}

/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// TnI pin config set if need	
	HAL_GPIO_ConfigureFunction(PCA, 6, PA6_MUX_T0IO);
	HAL_GPIO_ConfigOutput(PCA, 6, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCA, 6, DISABLE);
}


/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	T0_Config.CkSel = PCLK_2; // SystemPeriClock=48Mhz=PCLK  CLOCK_IN = PCLK/2=24Mhz 
	T0_Config.Prescaler = 23;    // TCLK = (CLOCK_IN / (PRS+1))  = 24Mhz / 24 = 1Mhz ->1us

	T0_Config.GRA = (500); // 0.5msec	DUTY = GRA
	T0_Config.GRB = (2000); // 2msec	PERIOD = GRB
	T0_Config.Cnt=0;		
//	T0_Config.StartLevel=START_LOW;
	T0_Config.StartLevel=START_HIGH;	
	T0_Config.AdcTrgEn=DISABLE;
	
	HAL_TIMER_Init(T0, ONESHOT_MODE, &T0_Config); // timer0 setting 
}


/**********************************************************************
 * @brief		TIMER_OneShotRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_OneShotRun(void)
{			
	HAL_TIMER_Cmd(T0, ENABLE); // timer start
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
 
	/*Configure port peripheral*/
	GPIO_Configure();

	/*Configure timer peripheral*/
	TIMER_Configure(); 
	
	/* Enable IRQ Interrupts */
	__enable_irq();
	
	/*TIMER Oneshot Test start*/
	TIMER_OneShotRun();
	
	while(1);
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
