/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/*
 U0: DMA mode
 U1: interrupt mode 
 
 for test, connect the following pins
 U0TX <-- --> U1RX
 U0RX <-- --> U1TX 
 
 check the buffer contents in debug 

*/


/* buffer size definition */
#define UART_RING_BUFSIZE 256

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void UART0_IRQHandler_IT(void);
void UART1_IRQHandler_IT(void);
void UART_IntErr(uint8_t bLSErrType);
void UART_IntTransmit(void);
void UART_IntReceive(void);

uint32_t UARTReceive(UART_Type *UARTx, uint8_t *rxbuf, uint8_t buflen);
uint32_t UARTSend(UART_Type *UARTx, uint8_t *txbuf, uint8_t buflen);

void U1Int(void);

void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void UART_DmaRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
__IO uint8_t  tx[UART_RING_BUFSIZE];  /*!< UART Tx data buffer */
__IO uint8_t  rx[UART_RING_BUFSIZE];  /*!< UART Rx data buffer */
	
__IO FlagStatus DMARxIntStat;
__IO FlagStatus DMATxIntStat;

static UART_CFG_Type UARTConfigStruct;	
static	uint32_t i;
static	DMAC_Ch_CFG_Type DMACU0Rx;
static	DMAC_Ch_CFG_Type DMACU0Tx;

static const uint8_t menu[] =
"************************************************\n\r"
" UART demo \n\r"
"\t - MCU: AC33M4064 \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 115200 bps \n\r"
" UART Dma test \n\r"
"************************************************\n\r";

/* Public Function ------------------------------------------------------------*/
/*********************************************************************//**
 * @brief		UART0 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void UART0_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp, tmp1;

	/* Determine the interrupt source */
	intsrc = U0->IIR;
	tmp = intsrc & UART_IIR_INTID_MASK;

	// Receive Line Status
	if (tmp == UART_IIR_INTID_RLS){
		// Check line status
		tmp1 = U0 ->LSR;
		// Mask out the Receive Ready and Transmit Holding empty status
		tmp1 &= (UART_LSR_OE | UART_LSR_PE | UART_LSR_FE | UART_LSR_BI);
	}

	if (tmp == UART_IIR_INTID_RDD){
		DMARxIntStat = SET;
	}
	
	if (tmp == UART_IIR_INTID_TDD){
		DMATxIntStat = SET;		
	}
}

/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}

/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
//UART0 pin config setting in PCU
	HAL_GPIO_ConfigureFunction(PCC, 14, PC14_MUX_RXD0);
	HAL_GPIO_ConfigOutput(PCC, 14, INPUT);
	HAL_GPIO_ConfigPullup(PCC, 14, ENABLE);
	
	HAL_GPIO_ConfigureFunction(PCC, 15, PC15_MUX_TXD0);
	HAL_GPIO_ConfigOutput(PCC, 15, PUSH_PULL_OUTPUT);	
	HAL_GPIO_ConfigPullup(PCC, 15, ENABLE);	
}


/**********************************************************************
 * @brief		UART_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_Configure(void)
{
	// default: 38400-8-N-1	
	HAL_UART_ConfigStructInit(&UARTConfigStruct);
	HAL_UART_Init(U0, &UARTConfigStruct);
	
	 /* Enable UART Rx DMA interrupt */
	HAL_UART_ConfigInterrupt((UART_Type *)U0, UART_INTCFG_DRX, ENABLE);
	/* Enable UART Tx DMA interrupt */
	HAL_UART_ConfigInterrupt((UART_Type *)U0, UART_INTCFG_DTX, ENABLE);

	HAL_UART_IFDelayConfig(U0, 1); // 2 bit time delay

  /* preemption = 1, sub-priority = 1 */
   NVIC_SetPriority(UART0_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for UART0 channel */
   NVIC_EnableIRQ(UART0_IRQn);

	U1Int();	
}

/**********************************************************************
 * @brief		DMA_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void DMA_Configure(void)
{
	HAL_DMAC_Init(); // DMAC peri enable 
}

/**********************************************************************
 * @brief		UART_DmaRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_DmaRun(void)
{			
	DMACU0Rx.TransferCnt = 10;
	DMACU0Rx.PeriSel = DMAC_PERISEL_U0RX;
	DMACU0Rx.TransferWidth = DMAC_WIDTH_BYTE;
	DMACU0Rx.Dir = DMAC_DIR_RX;
	DMACU0Rx.PeriAddr = 0x40008000;		
	DMACU0Rx.MemAddr = (uint32_t)(&rx[0]);
	HAL_DMAC_Setup(DC0, &DMACU0Rx);

	DMARxIntStat = RESET;		
	HAL_DMAC_ChCmd(DC0, ENABLE);

	UARTSend(U1,(uint8_t *)"1234567890",10);		
	
	while (DMARxIntStat == RESET) ;
	
	HAL_DMAC_ChCmd(DC0, DISABLE);
	DMARxIntStat = RESET;		

	for (i=0;i<10;i++){
		tx[i]=rx[i];
//			rx[i]=0;
	}
	tx[10]='\r';tx[11]='\n';

	DMACU0Tx.TransferCnt = 12;
	DMACU0Tx.PeriSel = DMAC_PERISEL_U0TX;
	DMACU0Tx.TransferWidth = DMAC_WIDTH_BYTE;
	DMACU0Tx.Dir = DMAC_DIR_TX;	
	DMACU0Tx.PeriAddr = 0x40008000;		
	DMACU0Tx.MemAddr = (uint32_t)(&tx[0]);	
	HAL_DMAC_Setup(DC1, &DMACU0Tx);	
	
	DMATxIntStat = RESET;
	HAL_DMAC_ChCmd(DC1, ENABLE);
	
	while (DMATxIntStat == RESET) ;
	
	HAL_DMAC_ChCmd(DC1, DISABLE);
	DMATxIntStat = RESET;		
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{	
	/*Configure dma peripheral*/
	DMA_Configure();
	
	/*Configure gpio peripheral*/
	GPIO_Configure();

	/*Configure uart peripheral*/
	UART_Configure(); 
	
	/* Enable IRQ Interrupts */
	__enable_irq();
	
	/*UART Dma Test start*/
	UART_DmaRun();
	
	while(1);
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
//	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
