/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void WDT_Configure(void);
void WDT_ResetRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
static WDT_CFG_Type wdtCfg;
static __IO uint32_t wdtcnt;
static uint8_t i,j;
static uint32_t msec;

static const uint8_t menu[] =
"************************************************\n\r"
"Watch dog timer Reset demo\n\r"
"\t - MCU: AC33M4064 \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 115200 bps \n\r"
"************************************************\n\r";

/* Public Function ------------------------------------------------------------*/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_IT(void) {
	if (msec) {
		msec--;
	}
}


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}

/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PB0
	HAL_GPIO_ConfigureFunction(PCB, 0, PB0_MUX_PB0);
	HAL_GPIO_ConfigOutput(PCB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCB, 0, ENABLE);	
	HAL_GPIO_SetPin(PB, _BIT(0));
}

/**********************************************************************
 * @brief		SYSTICK_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SYSTICK_Configure(void)
{
	// Enable Interrupt for Sytick timer
	NVIC_SetPriority (SysTick_IRQn, 1);
	
	//	1msec interrupt 	
	SysTick_Config(SystemCoreClock/1000); 
}


/**********************************************************************
 * @brief		WDT_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WDT_Configure(void)
{
	HAL_WDT_Init();

	wdtCfg.wdtResetEn = ENABLE;
	wdtCfg.wdtCountEn = DISABLE;
	wdtCfg.wdtClkSel = SET; //SET:external from MCCR3 (1Mhz ring OSC), RESET:PCLK
	wdtCfg.wdtPrescaler = WDT_DIV_1;  //WDTCLK = WDTCLKIN / PRS =  1Mhz / 1
	wdtCfg.wdtTmrConst = 1000000; //1s
	
	HAL_WDT_Configure(wdtCfg);
	
	HAL_WDT_Start(ENABLE);
	
	i=20;j=0;
}


/**********************************************************************
 * @brief		WDT_ResetRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WDT_ResetRun(void)
{			
	while(i){
		msec = 500;
		while(msec);

		HAL_WDT_UpdateTimeOut(1000000);
		i--;
		
		if (j==0){
			j=1;
			HAL_GPIO_ClearPin(PB, _BIT(0));
		}
		else {
			j=0;
			HAL_GPIO_SetPin(PB, _BIT(0));
		}
	}
	while(1);
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
 
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*Configure systick peripheral*/
	SYSTICK_Configure();
	
	/*Configure wdt peripheral*/
	WDT_Configure(); 
	
	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
		/*WDT Reset Test start*/
		WDT_ResetRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
